#include "LedControl.h" // MAXIM7219

const int bled = 3;    // Button LED 
const int button = 2;    // Big Red Button
const int led1 = 5;
const int led2 = 6;
const int led3 = 9;
int led1_value = 0;
int led2_value = 0;
int led3_value = 0;
long led1_end_time=0;
long led2_end_time=0;
long led3_end_time=0;

const int relay1 = A4;
const int relay2 = A3;

volatile int activated = 0;
volatile int host_notified = 0;

volatile int phase=5;
// phases - 0: activate, 1: countdown, 2 - relay1 on, 3 - relay2 on, 4 - relays off, 5 - waiting for button to be pressed
const long phase_length[] = {5000, 10000, 20000, 5000, 10000};
typedef void (* FuncPtr) (); 
FuncPtr PhaseAction[] = {ToActivate,ToCountdown,ToRelay1ON,ToRelay2ON,ToRelayOFF};

LedControl lc1=LedControl(12,11,10,1);

const unsigned long countdown_time = 30000;
unsigned long phase_time;
unsigned long activate_time;
unsigned long blink_time = 1500;


void setup()  { 
  pinMode(button, INPUT);           // set pin to input
  digitalWrite(button, HIGH);       // turn on pullup resistors
  pinMode(bled, OUTPUT);           // set pin to output

  pinMode(led1, OUTPUT);           // set pin to output
  pinMode(led2, OUTPUT);           // set pin to output
  pinMode(led3, OUTPUT);           // set pin to output

  pinMode(relay1, OUTPUT);           // set pin to output
  pinMode(relay2, OUTPUT);           // set pin to output

  
//        
  digitalWrite(bled, LOW); // light on

//  lc1.shutdown(0,true);
  digitalWrite(led1, HIGH);
  digitalWrite(led2, HIGH);
  digitalWrite(led3, HIGH);

  lc1.shutdown(0,false);
  lc1.clearDisplay(0);
  lc1.setIntensity(0, 8); // 0 to 15
//  lc1.setScanLimit(0, 8);
  
  ToRelayOFF();
  
  phase_time = millis();

  attachInterrupt(0, big_red_button_push, LOW);

  Serial.begin(9600); 
} 

void loop()  {
  if (phase!=5)
  {
      // check phase time
      unsigned long current_time=millis();
      if (current_time >= phase_time + phase_length[phase])
      {
        phase++;
        PhaseAction[phase]();
      }
    
      if (phase==1 || phase==2)
      {
        // indicator
        countdown();
      }
  }
  else if (activated == 1 && host_notified==0)
  {
     phase=0;
     PhaseAction[phase]();
  }

  BlinkLEDs();
}
// Interrupt Handler
void big_red_button_push()
{
  if (phase==5)
  {
    activated = 1;
    host_notified=0;
  }
}

void QuickBlinkButtonLED()
{
      unsigned long current_time=millis();
}

void SlowBlinkButtonLED()
{
      unsigned long current_time=millis();
}

void BlinkLEDs()
{
      unsigned long current_time=millis();
  // random
	int bled_value = TimeToLED(current_time, blink_time);
	analogWrite(bled, bled_value);
//        Serial.println(bled_value);
	  if (phase!=4 && phase!=5)
	  {
	    BlinkLED(current_time, led1, &led1_value, &led1_end_time);
	    BlinkLED(current_time, led2, &led2_value, &led2_end_time);
	    BlinkLED(current_time, led3, &led3_value, &led3_end_time);
	  }
}

void init_countdown()
{
  phase_time = millis();
  activate_time=phase_time;

  lc1.clearDisplay(0);

  lc1.setDigit(0, 7, 0, false);    
  lc1.setDigit(0, 6, 0, false);    

  lc1.setDigit(0, 4, 3, false);    
  lc1.setDigit(0, 3, 0, false);    

  lc1.setDigit(0, 1, 0, false);    
  lc1.setDigit(0, 0, 0, false);
  delay(1000);
  activate_time = millis(); // count 30 seconde from this momoent
}
void countdown_stop()
{
  lc1.setDigit(0, 7, 0, false);    
  lc1.setDigit(0, 6, 0, false);    

  lc1.setDigit(0, 4, 0, false);    
  lc1.setDigit(0, 3, 0, false);    

  lc1.setDigit(0, 1, 0, false);    
  lc1.setDigit(0, 0, 0, false);
}
void countdown()
{
  unsigned long current_time = millis();
  current_time -= activate_time;
  if (current_time >= countdown_time)
  {
    // stop the phase!
    countdown_stop();

  }
  current_time = countdown_time - current_time;
  unsigned long seconds = current_time/1000;
  unsigned long milliseconds = current_time%1000;
  milliseconds/=10;
  lc1.setDigit(0, 4, (byte)(seconds/10), false);
  lc1.setDigit(0, 3, (byte)(seconds%10), false);

  lc1.setDigit(0, 1, (byte)(milliseconds/10), false);
  lc1.setDigit(0, 0, (byte)(milliseconds%10), false);

}

void ToActivate()
{
    phase_time = millis();
    blink_time = 150;
    Serial.print("X");
    host_notified=1;
}
void ToCountdown()
{
    phase_time = millis();
    init_countdown();
}
void ToRelay1ON()
{
    phase_time = millis();
    digitalWrite(relay1, LOW);
}
void ToRelay2ON()
{
  phase_time = millis();
  countdown_stop();
  digitalWrite(relay2, LOW);     
}
void ToRelayOFF()
{
  blink_time = 1500;
  digitalWrite(relay1, HIGH);       
  digitalWrite(relay2, HIGH);      
  lc1.clearDisplay(0);
}
int TimeToLED(unsigned long a_time, unsigned long half_cycle) // half_cycle in ms, cannot be 0
{
	unsigned long normalized = a_time % (half_cycle + half_cycle);
	unsigned long y = (255 * normalized) / half_cycle;
	if (y>511) {
		y=0;
	} 
	else if (y>255)
	{
		y = 511-y;		
	}
	return (int)y;
}
int BlinkLED(long current_time, int a_led, int * a_value, long * a_end_phase)
{
	if (current_time > *a_end_phase)		
	{
		long time_to_next = random (200, 3000);
		*a_end_phase = current_time + time_to_next;
		if (*a_value==0)
		{
			*a_value = 1;
			digitalWrite(a_led,LOW);
		}
		else
		{
			*a_value = 0;
			digitalWrite(a_led,HIGH);
		}
	}
}
