﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.IO.Ports;
using System.Diagnostics;
using System.Threading;
using System.IO;

namespace DDD
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MediaPlayer mediaPlayer = new MediaPlayer();
        private BackgroundWorker worker;
        System.Windows.Threading.DispatcherTimer dispatcherTimer;
        public bool simulationMode = false;
        public int port = 11;
        volatile bool stopBanner = false;
        private Object logLock = new Object();

        public MainWindow()
        {
            InitializeComponent();

        }


        public double tb_x0
        {
            get { return (double)GetValue(tb_x0Property); }
            set { SetValue(tb_x0Property, value); }
        }

        // Using a DependencyProperty as the backing store for tb_x0.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty tb_x0Property =
            DependencyProperty.Register("tb_x0", typeof(double), typeof(MainWindow), new UIPropertyMetadata(0.0));


        public double tb_y0
        {
            get { return (double)GetValue(tb_y0Property); }
            set { SetValue(tb_y0Property, value); }
        }

        // Using a DependencyProperty as the backing store for tb_y0.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty tb_y0Property =
            DependencyProperty.Register("tb_y0", typeof(double), typeof(MainWindow), new UIPropertyMetadata(0.0));


        public double tb_x1
        {
            get { return (double)GetValue(tb_x1Property); }
            set { SetValue(tb_x1Property, value); }
        }

        // Using a DependencyProperty as the backing store for tb_x1.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty tb_x1Property =
            DependencyProperty.Register("tb_x1", typeof(double), typeof(MainWindow), new UIPropertyMetadata(0.0));


        public double tb_y1
        {
            get { return (double)GetValue(tb_y1Property); }
            set { SetValue(tb_y1Property, value); }
        }

        // Using a DependencyProperty as the backing store for tb_y1.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty tb_y1Property =
            DependencyProperty.Register("tb_y1", typeof(double), typeof(MainWindow), new UIPropertyMetadata(0.0));
       

        
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            mediaPlayer.Open(new Uri(@"c:\DDD\audio.mp3"));
            mediaPlayer.Volume = 0.2;
            mediaPlayer.MediaEnded += new EventHandler(mediaPlayer_MediaEnded);
            mediaPlayer.Pause();

            ToBanner();

        }

        void mediaPlayer_MediaEnded(object sender, EventArgs e)
        {
            if (Banner.Visibility == System.Windows.Visibility.Visible)
            {
                mediaPlayer.Position = TimeSpan.MinValue;
                mediaPlayer.Play();
            }
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            Log("Worker compete with " + ((int)(e.Result)).ToString());

            if ((int)(e.Result) == 1)
            {

                ToActivate();
                //            VideoControl.Source = new Uri(m_video);
            }
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {

            Log("DoWork");
            SerialPort mySerialPort = new SerialPort("COM"+port.ToString(),9600);
            mySerialPort.Parity = Parity.None;
            mySerialPort.StopBits = StopBits.One;
            mySerialPort.DataBits = 8;
            mySerialPort.Handshake = Handshake.None;

//            mySerialPort.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler);
            stopBanner = false;

            if (simulationMode)
            {
                Log("DoWork in Simulation");
                while (!stopBanner)
                {
                    Thread.Sleep(200); 
                }
                e.Result = 1;
            }
            else
            {

                try
                {
                    Log("DoWork before Open COM" + port.ToString());
                    mySerialPort.Open();

                    Log("DoWork After Open");

                    int intReturnASCII = 0;
                    while (mySerialPort.IsOpen && !stopBanner)
                    {
                        //Log("DoWork Inside While");
                        if (mySerialPort.BytesToRead > 0)
                        {
                            intReturnASCII = mySerialPort.ReadByte();
                            char ch = Convert.ToChar(intReturnASCII);
                            Log("DoWork Char " + ch);
                            if (ch == 'X')
                            {
                                break;

                            }
                        }
                    }
                    mySerialPort.Close();
                    e.Result = 1;
                    Log("DoWork Loop Done");
                }
                catch (Exception ex)
                {
                    Log("DoWork Exception "+ex.Message);
                    MessageBox.Show("COM communication error: " + ex.Message);
                    e.Result = 0;
                }
                finally
                {
                    if (mySerialPort.IsOpen)
                    {
                        mySerialPort.Close();
                    }
                }
            }
            Log("DoWork Done");
        }

        void InitializeBanner()
        {
            double width = Banner.ActualWidth;
            double height = Banner.ActualHeight;
            double margin = height / 3;
            tb_x0 = margin - D1.ActualWidth;
            tb_y0 = margin - D1.ActualHeight;
            tb_x1 = (width - margin) - D1.ActualWidth;
            tb_y1 = (height - margin) - D1.ActualHeight;


        }

        private void Banner_Loaded(object sender, RoutedEventArgs e)
        {
            InitializeBanner();
        }
        private void ToBanner()
        {
            Log("ToBanner Begin");
            VideoControl.Stop();
            VideoControl.Visibility = System.Windows.Visibility.Hidden;
            DDDActivated.Visibility = System.Windows.Visibility.Hidden;
            Banner.Visibility = System.Windows.Visibility.Visible;

            Log("ToBanner mediaPlayer.Play");
            mediaPlayer.Position = TimeSpan.MinValue;
            mediaPlayer.Play();

            Log("ToBanner Create Worker");
            worker = new BackgroundWorker();
            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
            worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(worker_RunWorkerCompleted);
            Log("ToBanner Run Worker");
            worker.RunWorkerAsync();

            Log("ToBanner End");
        }
        private void ToActivate()
        {
            Log("ToActivate Begin");
            VideoControl.Visibility = System.Windows.Visibility.Hidden;
            DDDActivated.Visibility = System.Windows.Visibility.Visible;
            Banner.Visibility = System.Windows.Visibility.Hidden;
            mediaPlayer.Pause();
            // start timer

            dispatcherTimer = new System.Windows.Threading.DispatcherTimer();
            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            dispatcherTimer.Interval = new TimeSpan(0, 0, 6);
            dispatcherTimer.Start();
            Log("ToActivate End");
        }

        void dispatcherTimer_Tick(object sender, EventArgs e)
        {
            dispatcherTimer.Stop();
            ToDestroy();
        }
        private void ToDestroy()
        {
            Log("ToDestroy Begin");
            VideoControl.Visibility = System.Windows.Visibility.Visible;
            DDDActivated.Visibility = System.Windows.Visibility.Hidden;
            Banner.Visibility = System.Windows.Visibility.Hidden;
            VideoControl.Play();
            Log("ToDestroy End");
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.System && e.SystemKey == Key.RightAlt)
            {
                stopBanner = true;
            }
            else if (e.Key == Key.RightCtrl)
            {
                if (worker.IsBusy)
                {
                    stopBanner = true;
                }
                mediaPlayer.Stop();
                VideoControl.Stop();
                if (dispatcherTimer != null)
                {
                    dispatcherTimer.Stop();
                }
                Thread.Sleep(2000);
                Close();
            }
            else if (e.Key == Key.X)
            {
                // Message from the 2nd Arduino!
                if (worker.IsBusy)
                {
                    stopBanner = true;
                }
            }
        }

        private void VideoControl_MediaEnded(object sender, RoutedEventArgs e)
        {
            ToBanner();
        }
        private void Log(string a_msg)
        {
            lock (logLock)
            {
                string dt = DateTime.Now.ToString("G");
                File.AppendAllText(@"c:\ddd\ddd_log.txt", dt + a_msg + "\n");
            }
        }

    }
}
